package com.example.nekontakte.users.api;

import com.example.nekontakte.core.api.PageAttributesMapper;
import com.example.nekontakte.core.configurations.Constants;
import com.example.nekontakte.users.model.UserEntity;
import com.example.nekontakte.users.service.UserService;

import jakarta.validation.Valid;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import java.util.Map;
import java.util.Objects;

@Controller

@RequestMapping(AdminUserController.URL)
public class AdminUserController {
    public static final String URL = Constants.ADMIN_PREFIX + "/users";
    private static final String USER_VIEW = "admin/users";
    private static final String USER_EDIT_VIEW = "admin/user";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String USER_ATTRIBUTE = "user";

    private final UserService userService;
    private final ModelMapper modelMapper;

    public AdminUserController(UserService userService, ModelMapper modelMapper) {
        this.modelMapper = modelMapper;
        this.userService = userService;
    }

    private UserDTO toDTO(UserEntity entity) {
        return modelMapper.map(entity, UserDTO.class);
    }

    @GetMapping
    public String getAll(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page, Model model) {
        final Map<String, Object> attributes = PageAttributesMapper
                .toAttributes(userService.getAll(page, Constants.DEFUALT_PAGE_SIZE), this::toDTO);
        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return USER_VIEW;
    }

    @GetMapping("/create")
    public String getCreateView(Model model) {
        model.addAttribute(USER_ATTRIBUTE, new UserEditDTO());
        return USER_EDIT_VIEW;
    }

    @GetMapping("/edit/{id}")
    public String getEditView(
            @PathVariable(name = "id") Integer id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }

        UserEntity userEntity = userService.get(id);
        UserEditDTO userEditDTO = new UserEditDTO();
        userEditDTO.setId(userEntity.getId());
        userEditDTO.setUsername(userEntity.getUsername());
        userEditDTO.setRole(userEntity.getRole());
        userEditDTO.setName(userEntity.getName());
        userEditDTO.setSurname(userEntity.getSurname());
        userEditDTO.setCity(userEntity.getCity());
        userEditDTO.setBirthday(userEntity.getBirthday());
        userEditDTO.setStatus(userEntity.getStatus());

        model.addAttribute(USER_ATTRIBUTE, userEditDTO);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return USER_EDIT_VIEW;
    }

    @PostMapping("/edit")
    public String postEditView(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = USER_ATTRIBUTE) @Valid UserEditDTO userEditDTO,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {

        UserEntity userEntity;

        if (userEditDTO.getId() == null) {
            userEntity = new UserEntity();
        } else {
            userEntity = userService.get(userEditDTO.getId());
        }

        userEntity.setUsername(userEditDTO.getUsername());
        userEntity.setRole(userEditDTO.getRole());
        userEntity.setName(userEditDTO.getName());
        userEntity.setSurname(userEditDTO.getSurname());
        userEntity.setCity(userEditDTO.getCity());
        userEntity.setBirthday(userEditDTO.getBirthday());
        userEntity.setStatus(userEditDTO.getStatus());

        if (userEditDTO.getPassword() != null && userEditDTO.getPassword() != "") {
            userEntity.setPassword(userEditDTO.getPassword());
        }

        if (userService.checkUserWithSameUsernameExist(userEntity)) {
            bindingResult.rejectValue("username", "userEdit:username", "Пользователь с таким логином уже есть");
        }

        if (userEditDTO.getId() == null && (userEditDTO.getPassword() == null || userEditDTO.getPassword() == "")) {
            bindingResult.rejectValue("password", "userEdit:passwords", "Укажите пароль");
        }

        if (!Objects.equals(userEditDTO.getPassword(), userEditDTO.getPasswordConfirm())) {
            bindingResult.rejectValue("passwordConfirm", "userEdit:passwords", "Пароли не совпадают");
        }

        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            model.addAttribute(USER_ATTRIBUTE, userEditDTO);
            return USER_EDIT_VIEW;
        }

        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);

        if (userEditDTO.getId() == null) {
            userService.create(userEntity);
        } else {
            userService.update(userEntity);
        }
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String postDeleteView(
            @PathVariable(name = "id") Integer id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        userService.delete(id);
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        return Constants.REDIRECT_VIEW + URL;
    }
}